@@ -7,9 +7,9 @@ except ImportError:
     from ordereddict import OrderedDict
     import simplejson as json
 
+import agate
 import six
 
-from csvkit import CSVKitWriter
 
 def geojson2csv(f, key=None, **kwargs):
     """
@@ -24,45 +24,51 @@ def geojson2csv(f, key=None, **kwargs):
         raise TypeError('JSON document is not valid GeoJSON: No top-level "type" key.')
 
     if js['type'] != 'FeatureCollection':
-        raise TypeError('Only GeoJSON with root FeatureCollection type is supported. Not %s' % js['type']) 
+        raise TypeError('Only GeoJSON with root FeatureCollection type is supported. Not %s' % js['type'])
 
     if 'features' not in js:
         raise TypeError('JSON document is not a valid FeatureCollection: No top-level "features" key.')
 
     features = js['features']
-    
+
     features_parsed = []    # tuples in the format (id, properties, geometry)
     property_fields = []
 
     for feature in features:
-        geoid = feature.get('id', None)
-
-        properties = feature.get('properties') or {}
+        properties = feature.get('properties', {})
 
         for prop in properties.keys():
             if prop not in property_fields:
                 property_fields.append(prop)
 
-        geometry = json.dumps(feature['geometry'])
+        geometry = feature['geometry']
+        geometry_type = geometry.get('type')
+        if geometry_type == 'Point':
+            longitude, latitude = geometry['coordinates']
+        else:
+            longitude, latitude = (None, None)
 
-        features_parsed.append((geoid, properties, geometry))
+        features_parsed.append((feature.get('id'), properties, json.dumps(geometry), geometry_type, longitude, latitude))
 
     header = ['id']
     header.extend(property_fields)
-    header.append('geojson')
+    header.extend(('geojson', 'type', 'longitude', 'latitude'))
 
     o = six.StringIO()
-    writer = CSVKitWriter(o)
+    writer = agate.csv.writer(o)
 
     writer.writerow(header)
 
-    for geoid, properties, geometry in features_parsed:
+    for geoid, properties, geometry, geometry_type, longitude, latitude in features_parsed:
         row = [geoid]
 
         for field in property_fields:
-            row.append(properties.get(field, None))
+            value = properties.get(field)
+            if isinstance(value, OrderedDict):
+                value = json.dumps(value)
+            row.append(value)
 
-        row.append(geometry)
+        row.extend((geometry, geometry_type, longitude, latitude))
 
         writer.writerow(row)
 
@@ -70,4 +76,3 @@ def geojson2csv(f, key=None, **kwargs):
     o.close()
 
     return output
-
