@@ -7,24 +7,8 @@ except ImportError:
 
 from csvkit import convert
 
+
 class TestConvert(unittest.TestCase):
-    def test_valid_file(self):
-        with open('examples/test.xls', 'rb') as f:
-            output = convert.convert(f, 'xls')
-        
-        with open('examples/testxls_converted.csv', 'r') as f:
-            self.assertEquals(f.read(), output)
-            
-    def test_no_file(self):
-        self.assertRaises(ValueError, convert.convert, None, 'xls')
-
-    def test_no_format(self):
-        with open('examples/dummy.csv', 'r') as f:
-            self.assertRaises(ValueError, convert.convert, f, None)
-
-    def test_invalid_format(self):
-        with open('examples/dummy.csv', 'r') as f:
-            self.assertRaises(ValueError, convert.convert, f, 'INVALID')
 
     def test_guess_fixed(self):
         self.assertEqual('fixed', convert.guess_format('testdata'))
@@ -32,9 +16,12 @@ class TestConvert(unittest.TestCase):
     def test_guess_xls(self):
         self.assertEqual('xls', convert.guess_format('testdata.xls'))
 
+    def test_guess_xls_uppercase(self):
+        self.assertEqual('xls', convert.guess_format('testdata.XLS'))
+
     def test_guess_xlsx(self):
         self.assertEqual('xlsx', convert.guess_format('testdata.xlsx'))
-    
+
     def test_guess_csv(self):
         self.assertEqual('csv', convert.guess_format('testdata.csv'))
 
@@ -44,3 +31,5 @@ class TestConvert(unittest.TestCase):
     def test_guess_json(self):
         self.assertEqual('json', convert.guess_format('testdata.json'))
 
+    def test_guess_invalid(self):
+        self.assertEqual(None, convert.guess_format('testdata.invalid'))
